// FLTK callback functions for butt
//
// Copyright 2007 by Daniel Noethen.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//

#include <errno.h>
#include <unistd.h>
#include <signal.h>

#ifndef _WIN32
 #include <sys/wait.h>
#endif

#include <FL/fl_ask.H>
#include <FL/Fl_File_Chooser.H>
#include <FL/Fl_Color_Chooser.H>

#include "cfg.h"
#include "butt.h"
#include "port_audio.h"
#include "timer.h"
#include "shoutcast.h"
#include "lame_encode.h"
#include "fl_callbacks.h"

#include "config.h"

flgui *fl_g = new flgui();
int display_info = STREAM_TIME;

void button_connect_cb()
{

    if(connected)
        return;

    int (*xc_connect)() = NULL;
    int rv;
    int dummy = 0;


    if(cfg.main.num_of_srv < 1)
    {
            print_info("There must be at least one server defined.\n"
                       "Click on the red button right to the LCD to\n"
                       "add a server", 1);
    return;
    }

    if(cfg.srv[cfg.selected_srv]->type == SHOUTCAST)
        xc_connect = &sc_connect;

    if(cfg.srv[cfg.selected_srv]->type == ICECAST)
    {
#if HAVE_LIBSHOUT
        xc_connect = &ic_connect;
#else
        print_info("butt compiled without icecast support", 1);
        return;
#endif
    }


    if(!strcmp(cfg.audio.codec, "ogg") && (cfg.audio.bitrate < 48))
    {
        print_info("ogg doesn't seem to support bitrates \n"
                    "lower than 48kbit",1);
        return;

    }
    if(!strcmp(cfg.audio.codec, "ogg") && (cfg.srv[cfg.selected_srv]->type == SHOUTCAST))
    {
        print_info("ShoutCast doesn't support ogg", 1);
        return;
    }



    char info_text_buf[128];

    if(cfg.main.num_of_srv < 1)
    {
        print_info("There must be at least one server defined.\n"
                   "Click on the red button right to the LCD to\n"
                   "add a server", 1);
        return;

    }
    if(cfg.srv[cfg.selected_srv]->type == SHOUTCAST)
        snprintf(info_text_buf, 127, "Connecting to %s:%u (%u) ...",
            cfg.srv[cfg.selected_srv]->addr,
            cfg.srv[cfg.selected_srv]->port+1,
            cfg.srv[cfg.selected_srv]->port);
    else
        snprintf(info_text_buf, 127, "Connecting to %s:%u ...",
            cfg.srv[cfg.selected_srv]->addr,
            cfg.srv[cfg.selected_srv]->port);

    print_info(info_text_buf, 0);

    try_to_connect = 1;

    while((rv = xc_connect()))
    {
        if( (rv == 2) || (try_to_connect == 0) )
        {
            try_to_connect = 0;  //invalid password or the user has pressed the stop button
            return;
        }

        switch(dummy)
        {

            case 0:
                print_lcd("connecting", 10, 0, 1);
                dummy++;
                break;
            case 1:
            case 2:
            case 3:
                print_lcd(".", 1, 0, 0);
                dummy++;
                break;
            case 4:
                dummy = 0;
                break;
            default:
                dummy++;
        }

        Fl::check();
    }

    if(connected)
    {
        print_info("connected", 0);

        //the user may not change the sound device while streaming
        fl_g->choice_cfg_dev->deactivate();
        //the sames applies to the codecs
        fl_g->radio_cfg_codec_mp3->deactivate();
        fl_g->radio_cfg_codec_ogg->deactivate();

        //Changing any audio settings while streaming does not work with ogg :(
        if(!strcmp(cfg.audio.codec, "ogg"))
        {
           fl_g->choice_cfg_bitrate->deactivate();
           fl_g->choice_cfg_samplerate->deactivate();
           fl_g->radio_cfg_channel_mono->deactivate();
           fl_g->radio_cfg_channel_stereo->deactivate();
        }

        fl_g->xrun->value(0);

        fl_g->xrun->value(0);
        fl_g->lcd->clear();
        fl_g->lcd->print((const uchar*)"info: idle", 10);


        pa_new_frames = 0;
        //timer_init(&stream_time);
        
        if(!recording)//We dont need to start this timeout if recording is running
            Fl::add_timeout(0.1, &check_time);

        Fl::add_timeout(0.05, &check_xrun);
        Fl::add_timeout(0.1, &check_if_disconnected);

        snd_start_stream();

        if(record && cfg.rec.start_rec)
        {
            snd_start_rec();
            timer_init(&rec_time);
        }

        display_info = STREAM_TIME;

    }
}

void button_cfg_cb()
{

    if(fl_g->window_cfg->shown())
    {
        fl_g->window_cfg->hide();
        fl_g->button_cfg->label("@>");

    }
    else
    {

/*
 * This is a bit stupid. Well, its Win32...
 * We need to place the cfg window a bit more to the right, otherwise
 * the main und the cfg window would overlap
*/

#ifdef _WIN32
    fl_g->window_cfg->position(fl_g->window_main->x() +
                                fl_g->window_main->w()+7,
                                fl_g->window_main->y());
#else
    fl_g->window_cfg->position(fl_g->window_main->x() +
                                fl_g->window_main->w(),
                                fl_g->window_main->y());
#endif
        fl_g->window_cfg->show();
        fl_g->button_cfg->label("@<");

        fill_cfg_widgets();
    }
}

void button_add_srv_ok_cb()
{

    int i;

    //error checking
    if(strlen(fl_g->input_add_srv_name->value()) == 0)
    {
        fl_alert("No name specified");
        return;
    }
    if(strlen(fl_g->input_add_srv_addr->value()) == 0)
    {
        fl_alert("No address specified");
        return;
    }
    if(strlen(fl_g->input_add_srv_pwd->value()) == 0)
    {
        fl_alert("No password specified");
        return;
    }
    if(strlen(fl_g->input_add_srv_port->value()) == 0)
    {
        fl_alert("No port specified");
        return;
    }
    else if(( atoi(fl_g->input_add_srv_port->value()) > 65535) ||
            (atoi(fl_g->input_add_srv_port->value()) < 0) )
    {
        fl_alert("Invalid portnumber\nThe portnumber may range from 0 to 65535");
        return;

    }

    i = cfg.main.num_of_srv;
    cfg.main.num_of_srv++;

    cfg.srv = (server_t**)realloc(cfg.srv, cfg.main.num_of_srv * sizeof(server_t*));
    cfg.srv[i] = (server_t*)malloc(sizeof(server_t));

    cfg.srv[i]->name = (char*)malloc(strlen(fl_g->input_add_srv_name->value())+1);
    sprintf(cfg.srv[i]->name, fl_g->input_add_srv_name->value());

    cfg.srv[i]->addr = (char*)malloc(strlen(fl_g->input_add_srv_addr->value())+1);
    sprintf(cfg.srv[i]->addr, fl_g->input_add_srv_addr->value());

    cfg.srv[i]->pwd = (char*)malloc(strlen(fl_g->input_add_srv_pwd->value())+1);
    sprintf(cfg.srv[i]->pwd, fl_g->input_add_srv_pwd->value());

    cfg.srv[i]->port = (unsigned int)atoi(fl_g->input_add_srv_port->value());

    if(fl_g->radio_add_srv_icecast->value())
    {
        cfg.srv[i]->mount = (char*)malloc(strlen(fl_g->input_add_srv_mount->value())+1);

        sprintf(cfg.srv[i]->mount, fl_g->input_add_srv_mount->value());
    }

    if(fl_g->radio_add_srv_shoutcast->value())
        cfg.srv[i]->type = SHOUTCAST;
    if(fl_g->radio_add_srv_icecast->value())
        cfg.srv[i]->type = ICECAST;

    if(cfg.main.num_of_srv > 1)
    {
        cfg.main.srv_ent = (char*)realloc(cfg.main.srv_ent,
                                         strlen(cfg.main.srv_ent) +
                                         strlen(cfg.srv[i]->name) +2);
        sprintf(cfg.main.srv_ent, "%s;%s", cfg.main.srv_ent, cfg.srv[i]->name);
    }
    else
    {
        cfg.main.srv_ent = (char*)malloc(strlen(cfg.srv[i]->name) +1);
        sprintf(cfg.main.srv_ent, "%s", cfg.srv[i]->name);
    }

    if(cfg.main.srv == NULL)
        cfg.main.srv = (char*)malloc(strlen(cfg.srv[cfg.selected_srv]->name)+1);
    else
        cfg.main.srv = (char*)realloc(cfg.main.srv,
                strlen(cfg.srv[cfg.selected_srv]->name)+1);

    sprintf(cfg.main.srv, cfg.srv[cfg.selected_srv]->name);

    //reset the input fields and hide the window
    fl_g->input_add_srv_name->value("");
    fl_g->input_add_srv_addr->value("");
    fl_g->input_add_srv_port->value("");
    fl_g->input_add_srv_pwd->value("");
    fl_g->input_add_srv_mount->value("");
    fl_g->window_add_srv->hide();

    unsaved_changes = 1;

    fill_cfg_widgets();

}

void button_cfg_del_srv_cb()
{

    int i;
    int diff;

    if(cfg.main.num_of_srv == 0)
        return;

    diff = cfg.main.num_of_srv-1 - cfg.selected_srv;

    for(i = 0; i < diff; i++)
    {
        memcpy(cfg.srv[cfg.selected_srv+i], cfg.srv[cfg.selected_srv+i+1],
                sizeof(server_t));

    }

    free(cfg.srv[cfg.main.num_of_srv-1]);


    cfg.main.num_of_srv--;
    cfg.selected_srv = 0;

    //rearrange the string that contains all server names
    memset(cfg.main.srv_ent, 0, strlen(cfg.main.srv_ent));
    for(i = 0; i < (int)cfg.main.num_of_srv; i++)
    {
        strcat(cfg.main.srv_ent, cfg.srv[i]->name);

        //the last entry doesn't have a trailing seperator ";"
        if(i < (int)cfg.main.num_of_srv-1)
            strcat(cfg.main.srv_ent, ";");

    }

    //clear the server choicemenu in the main and server section
    //fill_cfg_widgets() will refill them
    fl_g->choice_cfg_act_srv->clear();
    fl_g->choice_cfg_act_srv->redraw();       //Yes we need this :-(

    fl_g->choice_cfg_edit_srv->clear();
    fl_g->choice_cfg_act_srv->redraw();

    unsaved_changes = 1;

    fill_cfg_widgets();
}

void button_cfg_del_icy_cb()
{

    int i;
    int diff;

    if(cfg.main.num_of_icy == 0)
        return;

    diff = cfg.main.num_of_icy-1 - cfg.selected_icy;

    for(i = 0; i < diff; i++)
    {
        memcpy(cfg.icy[cfg.selected_icy+i], cfg.icy[cfg.selected_icy+i+1],
                sizeof(icy_t));

    }

    free(cfg.icy[cfg.main.num_of_icy-1]);

    cfg.main.num_of_icy--;
    cfg.selected_icy = 0;

     //rearrange the string that contains all ICY names
    memset(cfg.main.icy_ent, 0, strlen(cfg.main.icy_ent));
    for(i = 0; i < (int)cfg.main.num_of_icy; i++)
    {
        strcat(cfg.main.icy_ent, cfg.icy[i]->name);

        //the last entry doesn't have a trailing seperator ";"
        if(i < (int)cfg.main.num_of_icy-1)
            strcat(cfg.main.icy_ent, ";");

    }

    //clear the ICY choicemenu in the main and ICY section
    //fill_cfg_widgets() will refill them
    fl_g->choice_cfg_act_icy->clear();
    fl_g->choice_cfg_act_icy->redraw();           //Yes we need this :-(

    fl_g->choice_cfg_edit_icy->clear();
    fl_g->choice_cfg_act_icy->redraw();

    unsaved_changes = 1;

    fill_cfg_widgets();

}

void button_disconnect_cb()
{

    int button;

    if(try_to_connect)
    {
        if(recording)
        {
            button = fl_choice("stop recording?", "Cancel", "Yes", "No");
            switch(button)
            {

                case 0:
                    return;
                case 1:
                    snd_stop_rec();
                    fl_g->lcd->clear();
                    fl_g->lcd->print((const uchar*)"info: idle", 10);
            }
        }
        else
        {
            fl_g->lcd->clear();
            fl_g->lcd->print((const uchar*)"info: idle", 10);
        }

        fl_g->xrun->value(0);

        fl_g->choice_cfg_dev->activate();
        fl_g->radio_cfg_codec_mp3->activate();
        fl_g->radio_cfg_codec_ogg->activate();

        fl_g->choice_cfg_bitrate->activate();
        fl_g->choice_cfg_samplerate->activate();
        fl_g->radio_cfg_channel_mono->activate();
        fl_g->radio_cfg_channel_stereo->activate();

        try_to_connect = 0;

        Fl::remove_timeout(&check_xrun);
        Fl::remove_timeout(&check_if_disconnected);
        
        //only remove when not recording
        if(!recording)
            Fl::remove_timeout(&check_time);
        else
            display_info = REC_TIME;

        if(cfg.srv[cfg.selected_srv]->type == SHOUTCAST)
            sc_disconnect();
#if HAVE_LIBSHOUT
        else
            ic_disconnect();
#endif

        snd_stop_stream();

    }
}

void button_record_cb()
{


    int rc = 0, cancel = 0;
    char mode[3];

    if(record || recording)
    {
        snd_stop_rec();
        if(!connected)
        {
            fl_g->lcd->clear();
            fl_g->lcd->print((const uchar*)"info: idle", 10);
            Fl::remove_timeout(&check_time);
        }
        else
            display_info = STREAM_TIME;
    }
    else
    {

        cfg.rec.filename = fl_file_chooser("select file for recording",
           "mp3 files (*.mp3)\togg files (*.ogg)\tpcm data (*.pcm)", NULL);

        //check if a valid file was selected
        if(cfg.rec.filename != NULL)
        {
            //check if the file already exists
            if(fopen(cfg.rec.filename, "rb") != NULL)
            {
                rc = fl_choice("file already exists!",
                                "cancel", "overwrite", "append");

                switch(rc)
                {
                    case 0:                   //cancel pressed
                        cancel = 1;
                        break;
                    case 1:                   //overwrite pressed
                        sprintf(mode, "wb");
                        break;

                    case 2:                   //append pressed
                        sprintf(mode, "ab");
                }
            }
            else                              //selected file doesnt exist yet
                sprintf(mode, "wb");

            if(cancel == 0)
                if( (cfg.rec.fd = fopen(cfg.rec.filename, mode)) != NULL)
                {
                    record = 1;
                    if(!cfg.rec.start_rec || connected)
                    {
                        print_info("recording started", 0);

                        timer_init(&rec_time);
                        //create the recording thread
                        snd_start_rec();
                    }
                    else
                    {

                        print_info("recording starts when connected", 0);

                    }

                    if(!connected && !cfg.rec.start_rec)
                    {
                        display_info = REC_TIME;
                        Fl::add_timeout(0.1, &check_time);
                        timer_init(&rec_time);
                    }


                }
                else
                    fl_alert("could not open:\n%s", cfg.rec.filename);


        } //if(audio_settings.record.filename != NULL)

    }
}





void button_info_cb()
{

  if (!fl_g->info_visible)
  {
    // Show info output...
    fl_g->window_main->resize(fl_g->window_main->x(),
            fl_g->window_main->y(),
            fl_g->window_main->w(),
            fl_g->info_output->y() + 180);

    fl_g->info_output->show();
    fl_g->button_info->label("Info @2<");

    fl_g->info_visible = 1;

  }
  else
  {
    // Hide info output...
    fl_g->window_main->resize(fl_g->window_main->x(),
            fl_g->window_main->y(),
            fl_g->window_main->w(),
            fl_g->info_output->y());

    fl_g->info_output->hide();
    fl_g->button_info->label("Info @2>");
    fl_g->info_visible = 0;

  }
}


void choice_cfg_act_srv_cb()
{

    cfg.selected_srv = fl_g->choice_cfg_act_srv->value();

    cfg.main.srv = (char*)realloc(cfg.main.srv,
            strlen(cfg.srv[cfg.selected_srv]->name)+1);

    sprintf(cfg.main.srv, cfg.srv[cfg.selected_srv]->name);

    unsaved_changes = 1;
}
void choice_cfg_act_icy_cb()
{

    cfg.selected_icy = fl_g->choice_cfg_act_icy->value();

    cfg.main.icy = (char*)realloc(cfg.main.icy,
            strlen(cfg.icy[cfg.selected_icy]->name)+1);

    sprintf(cfg.main.icy, cfg.icy[cfg.selected_icy]->name);

    unsaved_changes = 1;
}


void button_cfg_add_srv_cb()
{

    fl_g->window_add_srv->position(fl_g->window_cfg->x(), fl_g->window_cfg->y());

    //give the "name" input field the input focus
    fl_g->input_add_srv_name->take_focus();

    fl_g->window_add_srv->show();


}
void button_cfg_add_icy_cb()
{

    fl_g->window_add_icy->position(fl_g->window_cfg->x(), fl_g->window_cfg->y());

    //give the "name" input field the input focus
    fl_g->input_add_icy_name->take_focus();

    fl_g->window_add_icy->show();


}

void button_cfg_song_go_cb()
{

    void (*xc_update_song)() = NULL;

    if(cfg.srv[cfg.selected_srv]->type == SHOUTCAST)
        xc_update_song = &sc_update_song;
    if(cfg.srv[cfg.selected_srv]->type == ICECAST)
    {
#if HAVE_LIBSHOUT
        xc_update_song = &ic_update_song;
#else
        print_info("butt compiled without icecast support", 1);
        return;
#endif
    }

    if(!connected)
    {
        print_info("You are not connected to a server", 1);
        return;
    }

     cfg.main.song =
        (char*)realloc(cfg.main.song,
                  strlen(fl_g->input_cfg_song->value())+1 );

    sprintf(cfg.main.song, fl_g->input_cfg_song->value());

    unsaved_changes = 1;

    xc_update_song();
}

void button_cfg_srv_apply_cb()
{

    if(cfg.main.num_of_srv < 1)
        return;

    int srv_num = fl_g->choice_cfg_edit_srv->value();

    //update current server address

    cfg.srv[srv_num]->addr =
        (char*) realloc(cfg.srv[srv_num]->addr,
                sizeof(char) * strlen(fl_g->input_cfg_addr->value())+1);

    sprintf(cfg.srv[srv_num]->addr, fl_g->input_cfg_addr->value());

    //update current server port
    cfg.srv[srv_num]->port = (unsigned int)atoi(fl_g->input_cfg_port->value());


    //update current server password

    cfg.srv[srv_num]->pwd =
        (char*) realloc(cfg.srv[srv_num]->pwd,
                    strlen(fl_g->input_cfg_passwd->value())+1);

    sprintf(cfg.srv[srv_num]->pwd, fl_g->input_cfg_passwd->value());

    //update current server type

    if(fl_g->radio_cfg_shoutcast->value())
        cfg.srv[srv_num]->type = SHOUTCAST;
    if(fl_g->radio_cfg_icecast->value())
        cfg.srv[srv_num]->type = ICECAST;

    //update current server mountpoint
    if(cfg.srv[srv_num]->type == ICECAST)
    {
        cfg.srv[srv_num]->mount =
            (char*) realloc(cfg.srv[srv_num]->mount,
                    sizeof(char) * strlen(fl_g->input_cfg_mount->value())+1);
        sprintf(cfg.srv[srv_num]->mount, fl_g->input_cfg_mount->value());
    }

    unsaved_changes = 1;


}

void button_cfg_icy_apply_cb()
{


    if(cfg.main.num_of_icy < 1)
        return;

    int icy_num = fl_g->choice_cfg_edit_icy->value();


    cfg.icy[icy_num]->desc =
        (char*)realloc(cfg.icy[icy_num]->desc,
                  strlen(fl_g->input_cfg_desc->value())+1 );
    sprintf(cfg.icy[icy_num]->desc, fl_g->input_cfg_desc->value());

    cfg.icy[icy_num]->genre =
        (char*)realloc(cfg.icy[icy_num]->genre,
                  strlen(fl_g->input_cfg_genre->value())+1 );
    sprintf(cfg.icy[icy_num]->genre, fl_g->input_cfg_genre->value());

    cfg.icy[icy_num]->url =
        (char*)realloc(cfg.icy[icy_num]->url,
                  strlen(fl_g->input_cfg_url->value())+1 );
    sprintf(cfg.icy[icy_num]->url, fl_g->input_cfg_url->value());

    cfg.icy[icy_num]->icq =
        (char*)realloc(cfg.icy[icy_num]->icq,
                  strlen(fl_g->input_cfg_icq->value())+1 );
    sprintf(cfg.icy[icy_num]->icq, fl_g->input_cfg_icq->value());

    cfg.icy[icy_num]->irc =
        (char*)realloc(cfg.icy[icy_num]->irc,
                  strlen(fl_g->input_cfg_irc->value())+1 );
    sprintf(cfg.icy[icy_num]->irc, fl_g->input_cfg_irc->value());

    cfg.icy[icy_num]->aim =
        (char*)realloc(cfg.icy[icy_num]->aim,
                  strlen(fl_g->input_cfg_aim->value())+1 );
    sprintf(cfg.icy[icy_num]->aim, fl_g->input_cfg_aim->value());

    sprintf(cfg.icy[icy_num]->pub, "%d", fl_g->check_cfg_pub->value());

    unsaved_changes = 1;

}

void choice_cfg_edit_srv_cb()
{

    char dummy[10];
    int server = fl_g->choice_cfg_edit_srv->value();


    fl_g->input_cfg_addr->value(cfg.srv[server]->addr);

    snprintf(dummy, 6, "%u", cfg.srv[server]->port);
    fl_g->input_cfg_port->value(dummy);
    fl_g->input_cfg_passwd->value(cfg.srv[server]->pwd);

    if(cfg.srv[server]->type == SHOUTCAST)
    {
        fl_g->input_cfg_mount->value("");
        fl_g->input_cfg_mount->deactivate();
        fl_g->radio_cfg_shoutcast->value(1);
        fl_g->radio_cfg_icecast->value(0);
    }
    if(cfg.srv[server]->type == ICECAST)
    {
        fl_g->input_cfg_mount->value(cfg.srv[server]->mount);
        fl_g->input_cfg_mount->activate();
        fl_g->radio_cfg_icecast->value(1);
        fl_g->radio_cfg_shoutcast->value(0);
    }
}

void choice_cfg_cat_cb()
{

    unsigned int bitrate[15] = { 32, 40, 48, 56, 64, 80, 96,
                                112, 128, 160, 192, 224, 256, 320, 0 };
    unsigned int i;


    if(fl_g->choice_cfg_cat->value() == 0) //streaming
    {

        if(!strcmp(cfg.audio.codec, "mp3"))
        {
            fl_g->radio_cfg_codec_mp3->value(1);
            fl_g->radio_cfg_codec_ogg->value(0);
        }
        else if(!strcmp(cfg.audio.codec, "ogg"))
        {
           fl_g->radio_cfg_codec_ogg->value(1);
           fl_g->radio_cfg_codec_mp3->value(0);
        }

        if(cfg.audio.channel == 1)
        {
            fl_g->radio_cfg_channel_mono->value(1);
            fl_g->radio_cfg_channel_stereo->value(0);
        }
        else if(cfg.audio.channel == 2)
        {
            fl_g->radio_cfg_channel_stereo->value(1);
            fl_g->radio_cfg_channel_mono->value(0);
        }


        for(i = 0; bitrate[i] != 0; i++)
            if(cfg.audio.bitrate == bitrate[i])
                fl_g->choice_cfg_bitrate->value(i);
#if HAVE_LIBVORBIS
        fl_g->radio_cfg_codec_ogg->activate();
#endif

    }
    else if(fl_g->choice_cfg_cat->value() == 1) //recording
    {

        if(!strcmp(cfg.rec.codec, "mp3"))
        {
            fl_g->radio_cfg_codec_mp3->value(1);
            fl_g->radio_cfg_codec_ogg->value(0);
        }
        else if(!strcmp(cfg.rec.codec, "ogg"))
        {
            fl_g->radio_cfg_codec_ogg->value(1);
            fl_g->radio_cfg_codec_mp3->value(0);
        }

        if(cfg.rec.channel == 1)
        {
            fl_g->radio_cfg_channel_mono->value(1);
            fl_g->radio_cfg_channel_stereo->value(0);
        }
        else if(cfg.rec.channel == 2)
        {
            fl_g->radio_cfg_channel_stereo->value(1);
            fl_g->radio_cfg_channel_mono->value(0);
        }


        for(i = 0; bitrate[i] != 0; i++)
            if(cfg.rec.bitrate == bitrate[i])
                fl_g->choice_cfg_bitrate->value(i);

        //ogg recording not implemented yet
        fl_g->radio_cfg_codec_ogg->deactivate();

    }
}

void choice_cfg_bitrate_cb()
{

    unsigned int sel_bitrate;
    unsigned int bitrate[14] = { 32, 40, 48, 56, 64, 80, 96,
                                112, 128, 160, 192, 224, 256, 320 };


    sel_bitrate = fl_g->choice_cfg_bitrate->value();



    if(fl_g->choice_cfg_cat->value() == 0)                //streaming
    {
        cfg.audio.bitrate = bitrate[sel_bitrate];

        if(cfg.main.num_of_icy > 0)
        {
            cfg.icy[cfg.selected_icy]->br = (char*)
                realloc(cfg.icy[cfg.selected_icy]->br,
                4*sizeof(char));
            snprintf(cfg.icy[cfg.selected_icy]->br, 4, "%d", cfg.audio.bitrate);
        }

    }
    else if (fl_g->choice_cfg_cat->value() == 1)          //recording
        cfg.rec.bitrate = bitrate[sel_bitrate];

    //this callback func gets called if the user changes the bitrate value
    //so we need to reinit the encode function with the new bitrate
    if(fl_g->radio_cfg_codec_mp3->value())
    {
        lame_enc_close();
        lame_enc_init();
    }
    /*
#if HAVE_LIBVORBIS
    if (fl_g->radio_cfg_codec_ogg->value())
    {
        vorbis_enc_close();
        vorbis_enc_init();
    }
#endif
*/
    unsaved_changes = 1;
}

void radio_cfg_channel_stereo_cb()
{
    if(fl_g->choice_cfg_cat->value() == 0)                //streaming
        cfg.audio.channel = 2;
    else if (fl_g->choice_cfg_cat->value() == 1)          //recording
        cfg.rec.channel = 2;

    snd_close();
    snd_init();
    snd_open_stream();

    if(fl_g->radio_cfg_codec_mp3->value())
    {

        lame_enc_close();
        lame_enc_init();
    }

#if HAVE_LIBVORBIS
    else
    {
        vorbis_enc_close();
        vorbis_enc_init();
    }
#endif



    unsaved_changes = 1;

}

void radio_cfg_channel_mono_cb()
{

    if(fl_g->choice_cfg_cat->value() == 0)                //streaming
        cfg.audio.channel = 1;
    else if (fl_g->choice_cfg_cat->value() == 1)          //recording
        cfg.rec.channel = 1;


    snd_close();
    snd_init();
    snd_open_stream();

    if(fl_g->radio_cfg_codec_mp3->value())
    {

        lame_enc_close();
        lame_enc_init();
    }

#if HAVE_LIBVORBIS
    else
    {
        vorbis_enc_close();
        vorbis_enc_init();
    }
#endif


    unsaved_changes = 1;

}



void button_add_srv_cancel_cb()
{

    fl_g->input_add_srv_name->value("");
    fl_g->input_add_srv_addr->value("");
    fl_g->input_add_srv_port->value("");
    fl_g->input_add_srv_pwd->value("");
    fl_g->input_add_srv_mount->value("");

    fl_g->window_add_srv->hide();

}

void button_add_icy_ok_cb()
{

    int i;
    //error checking
    if(strlen(fl_g->input_add_icy_name->value()) == 0)
    {
        fl_alert("No name specified");
        return;
    }

    i = cfg.main.num_of_icy;
    cfg.main.num_of_icy++;

    cfg.icy = (icy_t**)realloc(cfg.icy, cfg.main.num_of_icy * sizeof(icy_t*));
    cfg.icy[i] = (icy_t*)malloc(sizeof(icy_t));

    cfg.icy[i]->name = (char*)malloc(strlen(fl_g->input_add_icy_name->value())+1 );
    sprintf(cfg.icy[i]->name, fl_g->input_add_icy_name->value());

    cfg.icy[i]->desc = (char*)malloc(strlen(fl_g->input_add_icy_desc->value())+1 );
    sprintf(cfg.icy[i]->desc, fl_g->input_add_icy_desc->value());

    cfg.icy[i]->url = (char*)malloc(strlen(fl_g->input_add_icy_url->value())+1 );
    sprintf(cfg.icy[i]->url, fl_g->input_add_icy_url->value());

    cfg.icy[i]->genre = (char*)malloc(strlen(fl_g->input_add_icy_genre->value())+1 );
    sprintf(cfg.icy[i]->genre, fl_g->input_add_icy_genre->value());

    cfg.icy[i]->irc = (char*)malloc(strlen(fl_g->input_add_icy_irc->value())+1 );
    sprintf(cfg.icy[i]->irc, fl_g->input_add_icy_irc->value());

    cfg.icy[i]->icq = (char*)malloc(strlen(fl_g->input_add_icy_icq->value())+1 );
    sprintf(cfg.icy[i]->icq, fl_g->input_add_icy_icq->value());

    cfg.icy[i]->aim = (char*)malloc(strlen(fl_g->input_add_icy_aim->value())+1 );
    sprintf(cfg.icy[i]->aim, fl_g->input_add_icy_aim->value());

    cfg.icy[i]->pub = (char*)malloc(2 * sizeof(char));
    sprintf(cfg.icy[i]->pub, "%d", fl_g->check_add_icy_pub->value());

    cfg.icy[i]->br = (char*)malloc(4 * sizeof(char));
    sprintf(cfg.icy[i]->br, "%d", cfg.audio.bitrate);

     if(!strcmp(cfg.audio.codec, "mp3"))
     {
         cfg.icy[i]->type = (char*)malloc(strlen("audio/mpeg")+1);
         sprintf(cfg.icy[i]->type, "audio/mpeg");
     }


    if(cfg.main.num_of_icy > 1)
    {
        cfg.main.icy_ent = (char*)realloc(cfg.main.icy_ent,
                                         strlen(cfg.main.icy_ent) +
                                         strlen(cfg.icy[i]->name) +2);
        sprintf(cfg.main.icy_ent, "%s;%s", cfg.main.icy_ent, cfg.icy[i]->name);
    }
    else
    {
        cfg.main.icy_ent = (char*)malloc(strlen(cfg.icy[i]->name) +1);
        sprintf(cfg.main.icy_ent, "%s", cfg.icy[i]->name);
    }

    if(cfg.main.icy == NULL)
        cfg.main.icy = (char*)malloc(strlen(cfg.icy[cfg.selected_icy]->name)+1);
    else
        cfg.main.icy = (char*)realloc(cfg.main.icy,
                strlen(cfg.icy[cfg.selected_icy]->name)+1);

    sprintf(cfg.main.icy, cfg.icy[cfg.selected_icy]->name);

    fl_g->input_add_icy_name->value("");
    fl_g->input_add_icy_desc->value("");
    fl_g->input_add_icy_url->value("");
    fl_g->input_add_icy_genre->value("");
    fl_g->input_add_icy_irc->value("");
    fl_g->input_add_icy_icq->value("");
    fl_g->input_add_icy_aim->value("");
    fl_g->check_add_icy_pub->value(0);

    fl_g->window_add_icy->hide();

    fill_cfg_widgets();

    unsaved_changes = 1;

}

void button_add_icy_cancel_cb()
{

    fl_g->input_add_icy_name->value("");
    fl_g->input_add_icy_desc->value("");
    fl_g->input_add_icy_url->value("");
    fl_g->input_add_icy_genre->value("");
    fl_g->input_add_icy_irc->value("");
    fl_g->input_add_icy_icq->value("");
    fl_g->input_add_icy_aim->value("");
    fl_g->check_add_icy_pub->value(0);

    fl_g->window_add_icy->hide();

}




void choice_cfg_edit_icy_cb()
{

    int icy = fl_g->choice_cfg_edit_icy->value();

    fl_g->input_cfg_desc->value(cfg.icy[icy]->desc);
    fl_g->input_cfg_genre->value(cfg.icy[icy]->genre);
    fl_g->input_cfg_url->value(cfg.icy[icy]->url);
    fl_g->input_cfg_irc->value(cfg.icy[icy]->irc);
    fl_g->input_cfg_icq->value(cfg.icy[icy]->icq);
    fl_g->input_cfg_aim->value(cfg.icy[icy]->aim);

    if(!strcmp(cfg.icy[icy]->pub, "1"))
        fl_g->check_cfg_pub->value(1);
    else
        fl_g->check_cfg_pub->value(0);

}

void choice_cfg_dev_cb()
{

    cfg.audio.dev_num = fl_g->choice_cfg_dev->value();

    unsaved_changes = 1;

    snd_close();
    snd_init();
    snd_open_stream();

}

void radio_cfg_codec_mp3_cb()
{

    if(fl_g->choice_cfg_cat->value() == 0)    //streaming
        sprintf(cfg.audio.codec, "mp3");
    if(fl_g->choice_cfg_cat->value() == 1)    //recording
        sprintf(cfg.rec.codec, "mp3");

    unsaved_changes = 1;
}
void radio_cfg_codec_ogg_cb()
{
    if(fl_g->choice_cfg_cat->value() == 0)    //streaming
        sprintf(cfg.audio.codec, "ogg");
    if(fl_g->choice_cfg_cat->value() == 1)    //recording
        sprintf(cfg.rec.codec, "ogg");

    unsaved_changes = 1;
}

void ILM216_cb()
{
    if(Fl::event_button() == 1) //left mouse button
    {
        //change the display mode only when connected or recording
        //this will prevent confusing the user
        if(!connected && !recording)
            return;

        switch(display_info)
        {
            case STREAM_TIME:
                if(recording)
                    display_info = REC_TIME;
                else
                    display_info = SENT_DATA;
                break;

            case REC_TIME:
                if(connected)
                    display_info = SENT_DATA;
                else
                    display_info = REC_DATA;
                break;

            case SENT_DATA:
                if(recording)
                    display_info = REC_DATA;
                else
                    display_info = STREAM_TIME;
                break;

            case REC_DATA:
                if(connected)
                    display_info = STREAM_TIME;
                else
                    display_info = REC_TIME;


        }
    }
    if(Fl::event_button() == 3) //right mouse button
    {
        uchar r, g, b;

        Fl_Color bg, txt;
        bg  = (Fl_Color)cfg.main.bg_color;
        txt = (Fl_Color)cfg.main.txt_color;

        //Set the r g b values the color_chooser should start with
        r = (bg & 0xFF000000) >> 24;
        g = (bg & 0x00FF0000) >> 16;
        b = (bg & 0x0000FF00) >>  8;

        fl_color_chooser((const char*)"select background color", r, g, b);

        //The color_chooser changes the r, g, b, values to selected color
        cfg.main.bg_color = fl_rgb_color(r, g, b);

        fl_g->lcd->redraw();

        r = (txt & 0xFF000000) >> 24;
        g = (txt & 0x00FF0000) >> 16;
        b = (txt & 0x0000FF00) >>  8;

        fl_color_chooser((const char*)"select text color", r, g, b);
        cfg.main.txt_color = fl_rgb_color(r, g, b);

        fl_g->lcd->redraw();

        unsaved_changes = 1;
    }

}
