// audio functions for butt
//
// Copyright 2007 by Daniel Noethen.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <string.h>
#include <pthread.h>

#ifdef _WIN32
 #include <windows.h>
#endif

#include "butt.h"
#include "cfg.h"
#include "port_audio.h"
#include "parseconfig.h"
#include "lame_encode.h"
#include "shoutcast.h"

#include "config.h"


#define PA_FRAMES 2048
#define FRAMES_MUL 2
#define BUF_FRAMES PA_FRAMES*FRAMES_MUL

char* encode_buf;
short *pa_pcm_buf;
int buf_index;
int buf_size;
int buf_pos;
int framepacket_size;

int try_to_connect;
bool pa_new_frames;
bool snd_xrun;
bool reconnect;
int pa_callback_counter;
int encode_bytes_read;

pthread_t rec_thread;
pthread_t stream_thread;
pthread_mutex_t stream_mut, rec_mut;
pthread_cond_t  stream_cond, rec_cond;


PaStream *stream;

int snd_init()
{

    char info[100];

    PaError p_err;
    if(( p_err = Pa_Initialize()) != paNoError )
    {
        snprintf(info, 100, "PortAudio init failed:\n%s\n",
                            Pa_GetErrorText(p_err));
        ALERT(info);

        return 1;
    }
    reconnect = 0;
    buf_index = 0;

    return 0;
}

int snd_open_stream()
{

    int samplerate;

    PaDeviceIndex pa_dev_id;
    PaStreamParameters pa_params;
    PaError pa_err;
    const PaDeviceInfo *pa_dev_info;

    char info_text[128];

    if(cfg.audio.dev_count == 0)
    {
        print_info("ERROR: no sounddevice with input channels found", 1);
        return 1;
    }

    pa_callback_counter = 0;

    buf_size = BUF_FRAMES * 2;
    framepacket_size = PA_FRAMES * cfg.audio.channel;

    pa_pcm_buf = (short*)malloc(buf_size * sizeof(short));
    encode_buf = (char*)malloc(buf_size * sizeof(char));

    samplerate = cfg.audio.samplerate;

    if(cfg.audio.pcm_list[cfg.audio.dev_num]->dev_id == -1)
    {
        pa_dev_id = Pa_GetDefaultInputDevice();
        if(pa_dev_id == paNoDevice)
            pa_dev_id = 0;

    }
    else
        pa_dev_id = cfg.audio.pcm_list[cfg.audio.dev_num]->dev_id;
  
    pa_dev_info = Pa_GetDeviceInfo(pa_dev_id);
    if(pa_dev_info == NULL)
    {

        snprintf(info_text, 127, "Error getting device Info (%d)", pa_dev_id);
        print_info(info_text, 1);
        return 1;
    }

    pa_params.device = pa_dev_id;
    pa_params.channelCount = cfg.audio.channel;
    pa_params.sampleFormat = paInt16;
    pa_params.suggestedLatency = pa_dev_info->defaultHighInputLatency;
    pa_params.hostApiSpecificStreamInfo = NULL;

    sprintf(info_text, "Trying to open sound device:\n%s",
                       pa_dev_info->name);

    print_info(info_text, 0);

    pa_err = Pa_IsFormatSupported(&pa_params, NULL, samplerate);
    if(pa_err != paFormatIsSupported)
    {
        if(pa_err == paInvalidSampleRate)
        {
            sprintf(info_text,
                    "Samplerate not supported: %d\n"
                    "Trying default samplerate: %d",
                    samplerate, (int)pa_dev_info->defaultSampleRate);

            print_info(info_text, 1);
            if(Pa_IsFormatSupported(&pa_params, NULL,
                        pa_dev_info->defaultSampleRate) != paFormatIsSupported)
            {
                print_info("FAILED", 1);
                return 1;

            }
            else
            {
                samplerate = (int)pa_dev_info->defaultSampleRate;
                print_info("SUCCEED", 0);
            }
        }
        else
        {
            sprintf(info_text, "Format not supported. \n%s\n", Pa_GetErrorText(pa_err));
            print_info(info_text, 1);
            return 1;
        }
    }
    else
    {

        print_info("SUCCEED", 0);
    }


    pa_err = Pa_OpenStream(&stream, &pa_params, NULL,
                        samplerate, PA_FRAMES,
                        paNoFlag, snd_callback, NULL);

    if(pa_err != paNoError)
    {
        printf("error opening sound device: \n%s\n", Pa_GetErrorText(pa_err));
        return 1;

    }


    pthread_mutex_init(&stream_mut, NULL);
    pthread_cond_init (&stream_cond, NULL);

    pthread_mutex_init(&rec_mut, NULL);
    pthread_cond_init (&rec_cond, NULL);


    Pa_StartStream(stream);

    return 0;
}

void snd_start_stream()
{

    bytes_sent = 0;
    pthread_create(&stream_thread, NULL, snd_stream_thread, NULL);
}

void snd_stop_stream()
{

    connected = 0;
    print_info("user disconnected\n", 0);

}

void *snd_stream_thread(void *data)
{
    int (*xc_send)(char *buf, int buf_len) = NULL;
    short *local_audio_buf = (short*)malloc(buf_size*sizeof(short));

    encode_bytes_read = 0;

    if(cfg.srv[cfg.selected_srv]->type == SHOUTCAST)
    {
            xc_send = &sc_send;
    }
    if(cfg.srv[cfg.selected_srv]->type == ICECAST)
    {

#if HAVE_LIBSHOUT
        xc_send = &ic_send;
#else
        free(local_audio_buf);
        return;
#endif
    }

    while(connected)
    {
        pthread_cond_wait(&stream_cond, &stream_mut);
        pa_callback_counter = 0;

        memcpy(local_audio_buf, pa_pcm_buf, buf_size*sizeof(short));


        if(!strcmp(cfg.audio.codec, "mp3"))
            encode_bytes_read = lame_enc_buf(local_audio_buf, encode_buf,
                                    BUF_FRAMES, gfp_stream, buf_size);
#if HAVE_LIBVORBIS
        else
            encode_bytes_read = vorbis_enc_buf(local_audio_buf,
                                                      encode_buf,
                                                      BUF_FRAMES);
#endif

        if(xc_send(encode_buf, encode_bytes_read) == -1)
        {
            free(local_audio_buf);
            connected = 0; //disconnected
        }

        bytes_sent += encode_bytes_read;
    }

    free(local_audio_buf);
    pthread_exit(0);

}


void snd_start_rec()
{

    pthread_create(&rec_thread, NULL, snd_rec_thread, NULL);


    bytes_written = 0;
    recording = 1;

    print_info("recording to:", 0);
    print_info(cfg.rec.filename, 0);

}

void snd_stop_rec()
{

    fclose(cfg.rec.fd);
    record = 0;
    recording = 0;
    print_info("recording stopped", 0);

}

//The recording stuff runs in its own thread
//this prevents dropouts in the recording, in case the
//bandwidth is smaller than the selected streaming bitrate
void* snd_rec_thread(void *data)
{


    int   enc_bytes_read;
    char  *enc_buf = (char*)malloc(buf_size * sizeof(char));
    short *audio_buf = (short*)malloc(buf_size*sizeof(short));

    while(record)
    {
          pthread_cond_wait(&rec_cond, &rec_mut);

          memcpy(audio_buf, pa_pcm_buf, buf_size*sizeof(short));

          enc_bytes_read = lame_enc_buf(audio_buf, enc_buf,
                                BUF_FRAMES, gfp_record, buf_size);

          bytes_written += fwrite(enc_buf, 1, enc_bytes_read, cfg.rec.fd);
    }

    free(enc_buf);
    free(audio_buf);

    pthread_exit(0);

}

//this function is called by PortAudio
int snd_callback(const void *input,
                 void *output,
                 unsigned long frameCount,
                 const PaStreamCallbackTimeInfo* timeInfo,
                 PaStreamCallbackFlags statusFlags,
                 void *userData)
{
    //If this function gets more often called than FRAMES_MUL,
    //then the snd_send_buf function wasn fast enough to send the data to the server
    //this results in a buffer overrun
    if(pa_callback_counter++ >= FRAMES_MUL)
       snd_xrun = 1;
    else
       snd_xrun = 0;


    buf_pos = buf_index*framepacket_size;
    memcpy(pa_pcm_buf+buf_pos, input, framepacket_size*sizeof(short));

    pa_new_frames = 1;     //for the vu_update func

    buf_index++;
    if(buf_index == FRAMES_MUL)
    {
        if(connected)
            pthread_cond_signal(&stream_cond);
        if(recording)
            pthread_cond_signal(&rec_cond);

        buf_index = 0;
    }


    return 0;
}

void snd_update_vu() //Shows every 1024 recorded frames on the vu-meter
{
    short *p;
    p = pa_pcm_buf+buf_pos;

    //its not necessary to display every frame
    //Every 8/16th frame seems to be enough
    for(int i = 0; i < framepacket_size; i += cfg.audio.channel *8)
    {
        vu_meter(p[i], p[i+(cfg.audio.channel-1)]);
    }
    pa_new_frames = 0;
}

snd_dev_t **snd_get_devices(int *dev_count)
{

    int devcount, i, dev_num = 1;
    const PaDeviceInfo *p_di;
    char textbuf[100];
    char *p;

    snd_dev_t **dev_list;

    dev_list = (snd_dev_t**)malloc(100*sizeof(snd_dev_t*));

    for(i = 0; i < 100; i++)
        dev_list[i] = (snd_dev_t*)malloc(sizeof(snd_dev_t));


    dev_list[0]->name = (char*) malloc(strlen("Default PCM device (default)")+1);
    sprintf(dev_list[0]->name, "Default PCM device (default)");
    dev_list[0]->dev_id = -1;

    devcount = Pa_GetDeviceCount();
    if(devcount < 0)
    {
        snprintf(textbuf, 99, "PaError: %s", Pa_GetErrorText(devcount));
        print_info(textbuf, 1);
    }

    for( i = 0; i < devcount && i < 100; i++ )
    {
        p_di = Pa_GetDeviceInfo(i);
        if(p_di == NULL)
        {
            snprintf(textbuf, 127, "Error getting device Info_ (%d)", i);
            print_info(textbuf, 1);
            continue;
        }

        //Save only devices with input Channels
        if(p_di->maxInputChannels > 0)
        {
            dev_list[dev_num]->name = (char*) malloc(strlen(p_di->name)+1);
            sprintf(dev_list[dev_num]->name, p_di->name);
            dev_list[dev_num]->dev_id = i;

            while((p = strchr(dev_list[dev_num]->name, '/')) != NULL)
            {
                //replace every '/' with an '-'
                //otherweise FLTK will create a submenu for every '/' in the choice menu :-(
                *p = '-';
            }
            dev_num++;
        }
    }

    if(dev_num == 1)
        *dev_count = 0;
    else
        *dev_count = dev_num;

    return dev_list;

}


void snd_close()
{

    Pa_StopStream(stream);
    Pa_CloseStream(stream);
    Pa_Terminate();

    free(pa_pcm_buf);
    free(encode_buf);
}



